/* insert std header here */

#include "display_conf.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "commands.h"
#include "defines.h"
#include "log.h"
#include "misc.h" 
#include "output.h"

/* local globals */

static output_command_struct *output_command_head   = NULL;
static char                  *default_multi         = NULL; 
static char                  *default_single        = NULL; 

/* ------------------- LOCAL FUNCTIONS --------------------- */

/* find_command_len: finds the command len from the config
     file. Returns len if ok, negative number otherwise */
static int
find_command_len(command)
  char *command;
{
  output_command_struct *cur = output_command_head;

  while (cur != NULL)
  {
    if (cur->command != NULL && STR_EQ(cur->command, command))
    {
      if (cur->command_len) {
        return (cur->command_len);
      }
      else {
        return (strlen (command));
      }
    }
    cur = cur->nxt;
  }

  /* couldn't find the command */
  return(-1);
}

/* ------------------- PUBLIC FUNCTIONS --------------------- */

/* read_displays: reads the main configuration file. Returns TRUE if
      ok, FALSE otherwise */
int
read_displays(config_file)
  char *config_file;
{
  FILE                  *fp                         = NULL;
  char                  line[BUFSIZ];
  char                  tag[MAX_TEMPLATE_DESC];
  char                  datum[MAX_TEMPLATE_DESC];
  output_command_struct *cur                        = NULL;
  output_command_struct *new                        = NULL;

  if ((fp = fopen(config_file, "r")) == NULL)
  {
    log_error("cannot open displays file '%s': %s",
              config_file, l_strerror(errno));
    return FALSE;
  }

  while ((fgets(line, BUFSIZ, fp)) != NULL)
  {
    if (new_record(line) || output_command_head == NULL)
    {
      if ((new = NEW_OUTPUT_COMMAND) == NULL) {
        log_error("memory allocation problem");
        fclose(fp);
        return FALSE;
      }
      if (output_command_head == NULL)
      {
        cur                 = new;
        output_command_head = new;
      }
      else
      {
        cur->nxt = new;
        cur      = new;
      }
    }
    
    if (parse_line(line, tag, datum))
    {
      if (STR_EQ(tag, O_COMMAND))
      {
        cur->command = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_COMMAND_LEN))
      {
        cur->command_len = atoi (datum);
      }
      else if (STR_EQ(tag, O_MAGIC))
      {
        cur->magic = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_DESCRIPTION))
      {
        cur->description = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_HEADER))
      {
        cur->header = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_FOOTER))
      {
        cur->footer = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_RECORD_DELIM))
      {
        cur->record_delim = NEW_STRING(datum);
      }
      else if (STR_EQ(tag, O_DEFAULT_MULTI))
      {
        default_multi = NEW_STRING (datum);
      }
      else if (STR_EQ(tag, O_DEFAULT_SINGLE))
      {
        default_single = NEW_STRING (datum);
      }
      /* else ignore */
    }
  }
  
  /* hardcode dump command */

  if ((new = NEW_OUTPUT_COMMAND) == NULL)
  {
    log_error("memory allocation problem");
    fclose(fp);
    return FALSE;
  }
  if (output_command_head == NULL)
  {
    cur                 = new;
    output_command_head = new;
  }
  else
  {
    cur->nxt = new;
    cur      = new;
  }

  cur->command     = NEW_STRING("dump");
  cur->command_len = 4;
  cur->magic       = NEW_STRING("#");
  cur->description = NEW_STRING("display in tag/value format");

  /* add the display commands to the global command list */
  cur = output_command_head;
  while (cur != NULL)
  {
    if (cur->command != NULL)
    {
      append_command(cur->command, cur->command_len, cur->magic, NULL, TRUE, 
                     COMMAND_DISPLAY, NULL, NULL, NULL);
    }
    cur = cur->nxt;
  }

  fclose(fp);
  return TRUE;
}

/* read_display_format_from_object: reads the display command, etc
     from the template.  The expected format:
     command:length:filename */
output_display_struct *
read_display_format_from_object(top, str)
  output_display_struct *top;
  char                  *str;
{
  output_display_struct *new;
  output_display_struct *cur;
  char                  command[MAX_LINE];
  char                  filename[MAX_LINE];

  if (parse_line(str, command, filename))
  {
    new          = NEW_OUTPUT_DISPLAY;
    new->command = NEW_STRING(command);

    if ((new->command_len = find_command_len(command)) < 0)
    {
      /* command didn't have a length; so the whole command name is
         significant */
      new->command_len = strlen(command);
    }
    new->filename    = NEW_STRING(filename);
    new->format      = encode_output_format(new->format, new->filename);

    if (top == NULL)
    {
      return (new);
    }
    else
    {
      cur = top;
      while (cur->nxt != NULL)
      {
        cur = cur->nxt;
      }
      cur->nxt = new;
    }
  }

  return(top);
}

/* return_all_defined_displays: returns the complete list of displays */
output_command_struct *
return_all_defined_displays()
{
  return(output_command_head);
}

/* gets the display status */
char *
get_display_status(type)
  char *type;
{
  if (STR_EQ(type, MULTI_DISPLAY))
  {
    return(default_multi);
  }
  return(default_single);
}

/* Sets the display status -- assumes valid mode. Returns TRUE on
      success, FALSE on failure. */
int
set_display_status(type, mode)
  char  *type;
  char  *mode;
{
  char  **ptr;
  int   status;

  if (STR_EQ(type, ALL_DISPLAY))
  {
    status = set_display_status(MULTI_DISPLAY, mode);
    if (!status) return(status);
    
    status = set_display_status(SINGLE_DISPLAY, mode);
    return(status);
  }
  
  if (STR_EQ(type, MULTI_DISPLAY))
  {
    ptr = &default_multi;
  }
  else if (STR_EQ(type, SINGLE_DISPLAY))
  {
    ptr = &default_single;
  }
  else
  {
    return (FALSE);
  }
  free(*ptr);

  *ptr = NEW_STRING(mode);

  return (TRUE);
}

