/* insert std header here */

#include "fileutils.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <time.h>
#ifdef TIME_WITH_SYS_TIME
#include <sys/time.h>
#endif /* TIME_WITH_SYS_TIME */
#include <sys/stat.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "defines.h"
#include "log.h"
#include "strutil.h"


/* ----------------------- LOCAL STATICS ----------------------- */

static char cwd[MAX_FILE + 1];

/* ----------------------- LOCAL FUNCTIONS --------------------- */

static int
get_path_status(path, mode)
  char      *path;
  mode_t    *mode;
{
  struct stat   sb;
  int           status;
  
  status = stat(path, &sb);
  
  if (status < 0) {
    /* stat itself failed (file doesn't exist) */
    return FALSE;
  }

  *mode = sb.st_mode;
  return TRUE;
}

/* ----------------------- PUBLIC FUNCTIONS -------------------- */

/* file_exists: tests to see if a file exists.  Note: does not test
      for access. */
int
file_exists(file)
  char  *file;
{
  mode_t    mode;

  if (get_path_status(file, &mode) && S_ISREG(mode)) {
    return TRUE;
  }
  return FALSE;
}

int
directory_exists(dir)
  char  *dir;
{
  mode_t    mode;

  if (get_path_status(dir, &mode) && S_ISDIR(mode)) {
    return TRUE;
  }
  return FALSE;
}

int
is_rel_path(path)
  char  *path;
{
  if (path && *path != '/') {
    return TRUE;
  }
  return FALSE;
}

int
store_current_wd()
{
  bzero(cwd, sizeof(cwd));
  if (getcwd(cwd, sizeof(cwd)) == NULL)
  {
    log_error("store_current_wd: failed to get cwd: %s", l_strerror(errno));
    return FALSE;
  }

  return TRUE;
}   

int
restore_current_wd()
{
  if (chdir(cwd) < 0)
  {
    log_error("chdir to '%s' failed: %s", cwd, l_strerror(errno));
    return FALSE;
  }

  return TRUE;
}

time_t
get_path_mod_time(path)
  char  *path;
{
  struct stat   sb;
  int           status;
  
  status = stat(path, &sb);
  
  if (status < 0) {
    /* stat itself failed (file doesn't exist) */
    return FALSE;
  }

  return(sb.st_mtime);
}

/* path_full_to_rel: changes a full pathname to a path relative to the
      root.  Returns NULL if the directory wasn't in the root
      directory, otherwise, return a pointer to the reduced path
      within the full path */
char *
path_full_to_rel(full_path, root_dir)
  char *full_path;
  char *root_dir;
{
  int   root_dir_len  = strlen(root_dir);
  int   full_path_len = strlen(full_path);
  
  /* if the full path isn't long enough, immediately reject */
  if (full_path_len < root_dir_len) {
    return NULL;
  }

  /* check for the incorrect base */
  if (strncmp(full_path, root_dir, root_dir_len) != 0) {
    return NULL;
  }

  /* the full path is *exactly* the root directory */
  if (full_path_len == root_dir_len) {
    /* return a pointer to "" */
    return(full_path + root_dir_len);
  }
  return(full_path + root_dir_len + 1);
}

/* path_rel_to_full: basically just concatenates root_dir and
   rel_path; returns NULL if bad data.  It copies the original path
   into the new path if the 'rel_path' isn't actually relative. */
char *
path_rel_to_full(new_path, new_path_len, rel_path, root_dir)
  char *new_path;
  int  new_path_len;
  char *rel_path;
  char *root_dir;
{
  int   root_len;
  
  /* check for bad data */
  if (!new_path || !root_dir) {
    return NULL;
  }

  if (!is_rel_path(rel_path)) {
    strncpy(new_path, rel_path, new_path_len);
    return(new_path);
  }
  
  root_len = strlen(root_dir);

  bzero(new_path, new_path_len);
  
  strncat(new_path, root_dir, new_path_len);
  strncat(new_path, "/", new_path_len - root_len);
  strncat(new_path, rel_path, new_path_len - (root_len + 1));

  return(new_path);
}

/* canonicalize_path: this routine, given a path, will, if necessary,
      convert it to a relative path, and, if chrooted is TRUE, will
      return FALSE if the path isn't under the root directory. It will
      copy the new path into "new_path" */
int 
canonicalize_path(new_path, new_path_len, path, root_dir, chrooted,
                  null_allowed)
  char  *new_path;
  int   new_path_len;
  char  *path;
  char  *root_dir;
  int   chrooted;
  int   null_allowed;
{
  char      *p;
  
  /* first, reject if we are given bad input */
  if (!root_dir)
  {
    return FALSE;
  }

  if (!path)
  {
    if (null_allowed) return TRUE;
    return FALSE;
  }
  
  /* strip unecessary crap from path; assume root_dir has already had
     this done. */
  trim(path);
  if (strlen(path) > 1)
  {
    strip_trailing(path, '/');
  }

  /* check for the empty string */
  if (!*path)
  {
    if (null_allowed) return TRUE;
    return FALSE;
  }

  p = path;
  if (! is_rel_path(path))
  {
    /* convert to a relative path, if it can */
    p = path_full_to_rel(path, root_dir);
    if (! p)
    {
      /* not in root heirarchy */
      if (chrooted)
      {
        /* can't have that! */
        return FALSE;
      }
      /* otherwise, we just go back to the original full path */
      p = path;
    }
  }

  strncpy(new_path, p, new_path_len);
  return TRUE;
}

/*****************************************************************************
  Function: Returns current date/time in yymmddhhmmss string
  By: Mark Kosters
*****************************************************************************/
char *
make_timestamp()
{
  static char   buffer[15];
  struct tm     *tm;
  time_t        t;

  time(&t);
  tm = localtime(&t);

  strftime(buffer, 14, "%y%m%d%H%M%S", tm);

  return(buffer);
}

/*****************************************************************************
  Function: Returns a temp filename (that is also the ticket #
  By: Mark Kosters
*****************************************************************************/
char *
create_filename(fname, template, spool_directory)
  char *fname;
  char *template;
  char *spool_directory;

{
  sprintf(fname, template, spool_directory, make_timestamp());
  mktemp(fname);
  return(fname);
}

/*****************************************************************************
  Function: Returns a data text filename (that is also the ticket #
  By: Mark Kosters
*****************************************************************************/
char *
create_db_filename(fname, template, spool_directory, postfix)
  char *fname;
  char *template;
  char *spool_directory;
  char *postfix;
{
  sprintf(fname, template, spool_directory, make_timestamp(), postfix);
  mktemp(fname);
  return(fname);
}

