/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "register.h"

#include <stdio.h>
#include <string.h>

#include "defines.h"
#include "badref.h"
#include "client_msgs.h"
#include "fileutils.h"
#include "holdconnect.h"
#include "log.h"
#include "main.h"
#include "misc.h"
#include "objects.h"
#include "read_config.h"
#include "strutil.h"
#include "variables.h"


/* local defines */

#define SPOOL_FILE_TEMPLATE "%s/%s.XXXXXX"
#define DB_FILE_TEMPLATE    "%s/%s.XXXXXX"


/* local prototypes */

static int valid_registration_header PROTO((char *str, char *action,
                                            char *email_addres,
                                            char *authority_info));

static int valid_registration_on_mode PROTO((char *mode));

static int valid_registration_off_mode PROTO((char *mode));

static int valid_registration_action PROTO((char *action));

static int valid_registration_email_address PROTO((char *email));

static int register_off PROTO((char *line));

static void print_registration_header PROTO((FILE *fp, char *action,
                                             char *email_address));

static char *setup_update_program PROTO(());

static int object_in_registration PROTO((char *fname, char *object_name));

static void save_like_objects PROTO((char *fname, char *object_name,
                                     char *action, char *email_address,
                                     FILE *ofp));

/**************************************************************************
  records the registration stuff
**************************************************************************/
int
record_register(str)
  char *str;
{
  char          email_address[MAX_LINE];
  char          action[MAX_LINE];
  char          authority_info[MAX_LINE];
  char          line[MAX_LINE];
  char          fname[MAX_LINE];
  char          foname[MAX_LINE];
  char          params[MAX_LINE];
  int           loop                    = TRUE;
  int           multiple_registration   = FALSE;
  llist_struct  *object_list            = NULL;
  object_struct *object                 = NULL;
  FILE          *fp                     = NULL;
  char          *c                      = NULL;
  int           blanks                  = 0;
  int           run_bombed              = FALSE;

  if (!valid_registration_header(str, action, email_address, authority_info))
  {
    print_error(INVALID_DIRECTIVE_PARAM, "");
    return(FALSE);
  }

  if (!log_entry(get_register_log(), "register: %s", str))
  {
    print_error(UNIDENT_ERROR);
    return(0);
  }
  
  if (create_filename(fname, SPOOL_FILE_TEMPLATE,
                      get_register_spool()) == NULL ||
      (fp = fopen (fname, "w")) == NULL)
  {
    print_error(UNIDENT_ERROR);
    return(0);
  }

  print_ok();

  /* now suck in the registration information; nested directives will
     *not* work with this architecture */

  log_entry (get_register_log(), "file: %s", fname);
  do {
    /* read the line in and strip off trailing newline(s) */
    readline(line, MAX_LINE);
    stripchar(line, 13);

    /* if it is a -register off, end the read */
    if (register_off(line))
    {
      loop = FALSE;
    }
    else
    {
      /* otherwise, dump the line raw into the spool file */
      fprintf (fp, "%s\n", line);
    }

    /* if we got a blank line, we got multiple items in a single
       registration */
    if (! *line)
    {
      multiple_registration = TRUE;
      blanks++;
    }
    else
    {
      blanks = 0;
    }
  } while (loop && blanks < 10); /* only collect 10 registrations at
                                    once */

  fclose (fp);
      
  object_list = return_all_defined_objects("");

  while (object_list != NULL)
  {
    object = object_list->datum;
        
    if (object_in_registration(fname, object->name))
    {
      if ((create_filename(foname, DB_FILE_TEMPLATE,
                           object->dbdir) == NULL) ||
          strcat(foname, ".txt") == NULL || 
          (fp = fopen (foname, "w")) == NULL)
      {
        print_error(UNIDENT_ERROR,
                    "record_register open new registration file");
        run_bombed = TRUE;
      }
      else
      {
        /* transfer all the data for the same schema to this file
           located in the schema's data directory */
        save_like_objects(fname, object->name, action, email_address, fp);
        fclose (fp);

        /* and call the external parser */
        /* FIXME: this should really be internal */
        if ((c = get_config_file()) != NULL) {
          if (is_chrooted())
          {
            sprintf(params, "-q -j -c %s %s", c, foname);
          }
          else
          {
            sprintf(params, "-q -c %s %s", c, foname);
          }
        }
        else
        {
          strcpy(params, foname);
        }
/*         printf("RUN: %s %s\n", setup_update_program(), params); */
        
        if (!run_program(setup_update_program(), params))
        {
          run_bombed = TRUE;
        }
      }
    }
    object_list=object_list->nxt;
  } /* while */

  if (run_bombed)
  {
    /* FIXME: should print error here */
    return FALSE;
  }
  
  print_ok();
  return TRUE;
}

/* splits the directive parameter line into the its components;
   returns FALSE if the required parameters aren't present. */
static int
valid_registration_header(str, action, email_address, 
                          authority_info)
  char *str;
  char *action;
  char *email_address;
  char *authority_info;
{
  char *cp = str, *next, word[MAX_LINE];

  next = get_word(cp, word);
  if (!valid_registration_on_mode(word))
  {
    return (FALSE);
  }
  cp = next;
  next = get_word (cp, action);
  if (!valid_registration_action(action))
  {
    return (FALSE);
  }
  cp = next;
  next = get_word (cp, email_address);
  if (!valid_registration_email_address(email_address))
  {
    return (FALSE);
  }
  if (!*next)
  {
    return (TRUE); /* while this is technically against the RFC, we
                      will *not* require that auth-info be presented */
  }
  strcpy (authority_info, next);
  return (TRUE);
}

/* verify the "on" mode */
static int
valid_registration_on_mode(mode)
  char *mode;
{
  if (STR_EQ(mode, "on"))
  {
    return (TRUE);
  }

  return (FALSE);
}

/* verify the "off" mode */
static int
valid_registration_off_mode(mode)
  char *mode;
{
  if (STR_EQ(mode, "off"))
  {
    return (TRUE);
  }

  return (FALSE);
}

/* verify that the action is one of "add", "mod", and "del" */
static int
valid_registration_action(action)
  char *action;
{
  if (STR_EQ(action, "add") || STR_EQ(action, "mod") || STR_EQ(action, "del"))
  {
    return (TRUE);
  }

  return (FALSE);
}

/* verifies that the email address is an email address */
static int
valid_registration_email_address(email)
  char *email;
{
  /****
    put vrfy code here!
    ***/
  if (*email)
  {
    return (TRUE);
  }

  return (FALSE);
}

/**************************************************************************
  sees if should turn off registration mode
**************************************************************************/
static int
register_off(line)
  char *line;
{
  char  *next;
  char  word[MAX_LINE];

  if (*line != '-')
  {
    return (FALSE);
  }

  next = get_word(++line, word);
  if (STR_EQ(word, "register") || STR_EQ(next, "off"))
  {
    return (TRUE);
  }

  return (FALSE);
}

/**************************************************************************
  sets the registration header
**************************************************************************/
static void
print_registration_header(fp, action, email_address)
  FILE *fp;
  char *action;
  char *email_address;
{
  fprintf(fp, "version: %s\n", REGISTRATION_VERSION);
  fprintf(fp, "email: %s\n",   email_address);
  fprintf(fp, "action: %s\n",  action);
}

/**************************************************************************
  setup the update prog
**************************************************************************/
static char *
setup_update_program()
{
  static char   register_program[MAX_FILE];
  
  if (strlen(get_bin_path()))
  {
    sprintf(register_program, "%s/%s", get_bin_path(), get_update_prog());
  }
  else
  {
    strcpy(register_program, get_update_prog());
  }

  return(register_program);
}

/* Determines if the object can be determined from the registration
   data.  Either the registration data is of the form
   <object:attribute: data>, or there is an <object-type: [type]>
   line. */
static int
object_in_registration(fname, object_name)
  char *fname;
  char *object_name;
{
  FILE  *fp;
  char  line[MAX_LINE];
  char  attribute[MAX_LINE];
  char  datum[MAX_LINE];

  if ((fp = fopen(fname, "r")) == NULL)
  {
    log_error("object_in_registration: can not open %s: %s\n", fname,
              l_strerror(errno));
    return FALSE;
  }

  while (fgets(line, MAX_LINE, fp) != NULL)
  {
    stripchar(line, '\n');
    trim(line);

    if (*line)
    {
      parse_line(line, attribute, datum);

      /* check for the "object-type: <type>" style line */
      if ((STR_EQ(attribute, "object-type")) &&
          (STR_EQ(datum, object_name)))
      {
        fclose(fp);
        return TRUE;
      }

      /* or the object:attr:data style line */
      if (STR_EQ(attribute, object_name))
      {
        fclose(fp);
        return TRUE;
      }
    }
  }
    
  fclose (fp);
  return FALSE;
}

/**************************************************************************
  save all the same objects in fp
**************************************************************************/
static void
save_like_objects(fname, object_name, action, email_address, ofp)
  char  *fname;
  char  *object_name;
  char  *action;
  char  *email_address;
  FILE  *ofp;
{
  FILE  *ifp                    = NULL;
  int   got_object              = FALSE;
  int   multiple_regs           = FALSE;
  char  line[MAX_LINE];
  char  attribute[MAX_LINE];
  char  datum[MAX_LINE];

  if ((ifp = fopen(fname, "r")) == NULL)
  {
    log_error("save_like_objects: can not open %s: %s", fname,
              l_strerror(errno));
    return;
  }
  /* dump the version, email, and action into the new spool file */
  print_registration_header(ofp, action, email_address);

  while (fgets(line, MAX_LINE, ifp) != NULL)
  {
    stripchar(line, '\n');
    trim(line);

    /* if we have encountered a blank line, and we have already
       determined the object, then set the correct flags */
    if (! *line)
    {
      if (got_object)
      {
        multiple_regs = TRUE;
        got_object = FALSE;
      }

      continue;
    }

    parse_line(line, attribute, datum);
    
    /* if we have the <object:attribute: data> style line... */
    if (STR_EQ(attribute, object_name))
    {
      /* put a separator in for rwhoisparse */
      if (multiple_regs && !got_object)
      {
        fprintf(ofp, "\n");
      }
      
      fprintf(ofp, "%s:%s\n", attribute, datum);
      got_object = TRUE;
    }
    else
    {
      /* we just have a <attribute>: <data> style line */

      /* deal with the separator */
      if (multiple_regs && !got_object)
      {
        fprintf(ofp, "\n");
      }

      /* skip the Object-Type: lines */
      if (STR_EQ(attribute, "object-type"))
      {
        continue;
      }
      
      fprintf(ofp, "%s:%s:%s\n", object_name, attribute, datum);
      got_object = TRUE;
    }
  }
  
  fclose(ifp);
}
