/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "whois.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "attributes.h"
#include "cache.h"
#include "client_msgs.h"
#include "debug.h"
#include "defines.h"
#include "directive.h"
#include "displays.h"
#include "forward.h"
#include "help.h"
#include "holdconnect.h"
#include "limit.h"
#include "log.h"
#include "mkdb_interface.h"
#include "objects.h"
#include "parse.h"
#include "records.h"
#include "reduce.h"
#include "strutil.h"
#include "variables.h"

/* local prototypes */

static int can_do_more_searches PROTO((int               hits,
                                       int               pass,
                                       object_type       *search_object_type,
                                       int               *reduce,
                                       int               *action,
                                       query_item_struct *query_list,
                                       int               internal_see_also));

static record_struct *run_search PROTO((object_type       search_object_type,
                                        int               cache,
                                        int               pass,
                                        query_item_struct *query_list));

static record_struct *get_root_server_records PROTO(());


/* whois: this is the main ball of wax.

   Returns TRUE if the connection should *not* be terminated
   (directive or query with holdconnect on), returns FALSE otherwise.

   pass 0 is for regular and cached data
   pass 1 is for referral (no reduction)
   pass 2+ is for referrals with reductions */

int
whois(query_str, root_server, internal_see_also)
  char *query_str;
  int  root_server;
  int  internal_see_also;

{
  record_struct     *records;
  query_item_struct *query_list;
  int               reduce      = FALSE;
  int               pass        = 0;
  int               hits        = FALSE;
  int               next_action;
  object_type       search_object_type;
  int               status      = FALSE;
  
  trim(query_str);
  log_entry (RWHOIS_LOG_FILE, "Pass 0 Arg: %s", query_str);
  save_original_query(query_str);

  /* okay this is for those who type return or other bad things */
  if (!query_str || !*query_str ||
      (query_list = encode_command(query_str)) == NULL)
  {
    help();
    return(holdconnect_status());
  }

  /* this is for the directives only */

  if (is_directive(query_list))
  {
    status = run_directive(query_list);
    if (status == QUIT)
    {
      return FALSE;
    }
    return TRUE;
  }

  /* does the search/display for search commands */

  while (can_do_more_searches(hits, pass, &search_object_type, &reduce, 
                              &next_action, query_list, internal_see_also))
  {
    /* this is kind of a hack, but it seems necessary to put the
       ip-network looking things into a standard format so that they
       get found the first time round, if possible. */
    {
      query_item_struct *ql = query_list;

      while (ql != NULL)
      {
        is_ip_network(ql, pass, 0);
        ql = ql->nxt;
      }
    }
      
    if (reduce)
    {
      reduce_query(query_list, pass);
    }
    hits = run_query(search_object_type, FALSE, pass, query_list,
                     internal_see_also);
    record_results (FALSE, reduce, hits);
    if (cache_status())
    {
      int old_hits = hits;
      hits += run_query(OBJECT, TRUE, pass, query_list, internal_see_also);
      record_results(TRUE, FALSE, hits-old_hits);
    }

    pass++;
  } 

  /* Done with the searching - now do what next_action says to do */

  switch (next_action)
  {
  case PUNT_ACTION:
    if (!root_server)
    {
      if ((records = get_root_server_records()) != NULL)
      {
        log_entry(RWHOIS_LOG_FILE, "ROOT QUERY");
      }
      display_results(records, FALSE, "summary", NULL, FALSE);
      print_ok();
    }
    else if (!hits)
    {
      print_error(NO_RECORDS_FOUND, "");
    }
    break;

  case WHOIS_PP_ACTION:
    printf("%%info on\n");
    printf("This query is not in our database and appears to be non-reducible\n");
    printf("-- OR --\n");
    printf("we do not have a referral for this information.\n");
    printf("%%info off\n");
    print_error(NO_RECORDS_FOUND, "");
    break;

  case DONE_ACTION:
    if (!internal_see_also)
    {
      print_ok();
    }
    break;
  }
  if (!internal_see_also)
  {
    return(holdconnect_status());
  }
  else
  {
    return(hits);
  }
}

/******************************************************************************
  this procedure sees if ok to keep on calling the search loop
  returns TRUE to keep on going
          FALSE otherwise
******************************************************************************/
static int
can_do_more_searches(hits, pass, search_object_type, reduce, 
                     action, query_list, internal_see_also)
  int               hits;
  int               pass;
  object_type       *search_object_type;
  int               *reduce;
  int               *action;
  query_item_struct *query_list;
  int               internal_see_also;
{
  /* for now, if any hits - exit out */
  if (hits > 0)
  {
    *action = DONE_ACTION;
    return (FALSE);
  }
  
  /* for now, leave this be */
  if (pass > 0)
  {
    if (internal_see_also)
    {
      *action = DONE_ACTION;
      return (FALSE);
    }
    *reduce =  TRUE;
    *search_object_type = REFERRAL;
    if ((*action = is_query_reducible (query_list, pass)) != KEEPON_ACTION)
    {
      return (FALSE);
    }
  }
  else
  {
    *search_object_type = OBJECT;
  }

  return (TRUE);
}

/******************************************************************************
  this procedure does query - calling the search and display modules
  returns number of hits
******************************************************************************/
int
run_query(search_object_type, cache, pass, query_list, sub_query)
  object_type       search_object_type;
  int               cache;
  int               pass;
  query_item_struct *query_list;
  int               sub_query;

{
  record_struct *records;
  int           hits;

  records = run_search(search_object_type, cache, pass, query_list);

  hits    = display_results(records, cache, get_display_mode(query_list), 
                            decode_data_for_authority_element (query_list),
                            sub_query);

  if (hits)
  {
    free_records(records);
  }

  return (hits);
}

/******************************************************************************
  this procedure runs the search part of the query
******************************************************************************/
static record_struct *
run_search(search_object_type, cache, pass, query_list)
  object_type       search_object_type;
  int               cache;
  int               pass;
  query_item_struct *query_list;
{
  llist_struct  *list_of_objects;
  char          query_str[MAX_LINE];
  object_struct *object;
  record_struct *records        = NULL;
  char          *db_fname;
  int           mkdb_error_code = 0;

  list_of_objects = setup_search_objects(query_list, search_object_type);
  decode_command_list(query_str, query_list);

  if (pass > 0)
  {
    log_entry (RWHOIS_LOG_FILE, "Pass %d Arg: %s", pass, query_str);
  }

  while (list_of_objects != NULL)
  {
    object = list_of_objects->datum;
    if (cache && object->cache_db != NULL)
    {
      db_fname = object->cache_db;
    }
    else
    {
      db_fname = object->local_db;
    }
    if (debug_status())
    {
      printf ("run_search: trying template |%s| file |%s|\n", object->name,
              db_fname);
      printf ("run_search: looking for |%s|\n", query_str);
    }
    records = mkdb_interface(db_fname, query_str, get_max_hit_limit(), 
                             (search_object_type ==
                              REFERRAL ? MAX_REFERRAL_LINES : 0),
                             &mkdb_error_code, records);
    list_of_objects = list_of_objects->nxt;
  }
  return (records);
}

/******************************************************************************
  this procedure runs the search part of the query
******************************************************************************/
void
record_results (cache, reduce, hits)
  int cache;
  int reduce;
  int hits;

{
  if (cache)
  {
    log_entry (RWHOIS_LOG_FILE, "Ans: %d cache hits", hits);
  }
  else if (reduce)
  {
    log_entry (RWHOIS_LOG_FILE, "Ans: %d referral hits", hits);
  }
  else
  {
    log_entry (RWHOIS_LOG_FILE, "Ans: %d data hits", hits);
  }
}

/******************************************************************************
  this procedure returns the root server record
  now it is hard-coded - this will change
  bug fixes thanks to drc@nis.apnic.net 4/29/94
******************************************************************************/
static record_struct *
get_root_server_records()

{
  char          txt[MAX_LINE*5];
  char          line[MAX_LINE];
  record_struct *top    = NULL;
  record_struct *new;
  record_struct *cur    = NULL;
  FILE          *fp;

  *txt = '\0';

  if ((fp = fopen((char *)get_root_file(), "r")) == NULL)
  {
    print_error (MISC_DIAG, "Can not open root server file '%s'", 
                 get_root_file());
  }

  while ( fgets(line, MAX_LINE, fp) != NULL )
  {
    /* if end of root record */
    if (strncmp (line,"-----", 5) == 0)
    {
      if (*txt != '\0')
      {
        new = NEW_RECORD;
        new->fields = read_fields_from_text (txt);
        if (top == NULL)
        {
          cur = top = new;
        }
        else
        {
          cur->nxt = new;
          cur = new;
        }
        *txt = '\0';
      }
    }
    else
    {
      strncat(txt, line, MAX_LINE*5 - strlen(txt));
    }
  }

  fclose (fp);

  if (*txt != '\0')
  {
    new = NEW_RECORD;
    new->fields = read_fields_from_text (txt);
    if (top == NULL)
    {
      cur = top = new;
    }
    else
    {
      cur->nxt = new;
      cur = new;
    }
  }

  return (top);
}
