/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/

/**************************************************************************
  Change history

9/13/94 Included additional col in build_unsorted_index to mention that 
        element deleted (TRUE) or not (FALSE).
***************************************************************************/
#include "indexer.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "defines.h"
#include "fileutils.h"
#include "io.h"
#include "log.h"
#include "misc.h"
#include "read_config.h"
#include "rwhois_supp.h"
#include "strutil.h"
#include "variables.h"

#define DATA_FILE             "DATA"
#define INDEX_FILE            "INDEX"

/* local prototypes */

static void usage PROTO((char *prog_name));


void
main(argc, argv)
  int argc;
  char *argv[];
{
  extern int    optind;
  extern char   *optarg;
  char          c;
  int           err             = 0;
  int           in_jail         = FALSE;
  int           mode            = 0;
  int           type            = AUTHORITATIVE;
  char          *config_file    = NULL;
  char          *file_name      = NULL;
  char          current_path[MAX_FILENAME];
  char          tmp_path[MAX_FILENAME];
  char          tmp_rel_path[MAX_FILENAME];
  
  /* inititalize config file data */
  init_config_data();

  /* save our current path, so we can correctly handle command line
     relative paths */
  getcwd(current_path, sizeof(current_path));
  
  /* command line parsing */
  while ((c = getopt(argc, argv, "c:m:t:qjved")) != EOF) {
    switch (c) {
    case 'c': /* config file */
      config_file = optarg;
      break;
    case 'd': /* debug */
      set_verbosity(3);
      break;
    case 'm': /* mode options i for initialize a for append */
      if (STRN_EQ(optarg, "i", 1))
        mode = INITIALIZE;
      else if (STRN_EQ(optarg, "a", 1))
        mode = APPEND;
      else {
        log_error("invalid mode: <a|i>");
        err = 1;
      }
      break;
    case 't': /* type options c for cached a for authoritive */
      if (STRN_EQ(optarg, "c", 1))
        type = CACHED;
      else if (STRN_EQ(optarg, "a", 1))
        type = AUTHORITATIVE;
      else {
        log_error("invalid type <a|c>");
        err = 1;
      }
      break;
    case 'j': /* in jail so we are running in a chrooted enviroment */
      in_jail = TRUE;
      break;
    case 'e': /* rwhoisd will run in a chrooted environment */
      set_ensure_in_jail(TRUE);
      break;
    case 'q': /* quiet (don't spew out messages) */
      set_verbosity(0);
      break;
    case 'v': /* be extra verbose */
      set_verbosity(2);
      break;
    default:
      usage(argv[0]);
      break;
    }
  }

  /* required ops missing */
  if (mode == 0 && !err)
  {
    log_error("missing mode: should be -mi or -ma");
    usage(argv[0]);
  }
  if (argc == 1 || err)
  {
    usage(argv[0]);
  }

  /* read in standard rwhois configuration */
  
  if (config_file == NULL)
  {
    config_file = DEFAULT_RWHOIS_CONFIG_FILE;
  }
  if (!read_schema_config_files(config_file, in_jail))
  {
    log_error("error opening configuration files.  Exiting");
    exit(64);
  }

  /* show the command line status */
  if (get_verbosity()) {
    fprintf(stderr, "-------------------\n");
    if (mode == INITIALIZE)
    {
      fprintf(stderr, "mode: initialize\n");
    }
    else if (mode == APPEND)
    {
      fprintf(stderr, "mode: append\n");
    }
    if (type == AUTHORITATIVE)
    {
      fprintf(stderr, "type: authoritative\n");
    }
    else
    {
      fprintf(stderr, "type: cached\n");
    }
    fprintf(stderr, "config_file: %s\n", config_file);
    fprintf(stderr, "verbosity_level: %d\n", get_verbosity());
    if (in_jail)
    {
      fprintf(stderr, "state: indexer running chrooted\n");
    }
    if (get_ensure_in_jail())
    {
     fprintf(stderr, "state: rwhoisd will run chrooted\n");
    }
    fprintf(stderr, "-------------------\n");
  }

  /* we are probably already here, but lets make sure */
  chdir_root_dir();
    
  /* the rest of the command line should be a list of files */
  while (optind < argc)
  {
    file_name = argv[optind];

    /* paths: we would like to keep all paths relative to the root
       (default-dir) if possible.  Then, based on 'ensure_in_jail' we
       can expand to a full path correctly.  If the path remains a
       full path, then it is outside of the root, and will fail if
       'ensure_in_jail' */

    /* relative paths are probably given to us relative to the
       original working directory */
    if  (is_rel_path(file_name))
    {
      sprintf(tmp_path, "%s/%s", current_path, file_name);
      file_name = tmp_path;
    }

    /* now make them relative to the rwhois-root-dir, if possible */
    if (! canonicalize_path(tmp_rel_path, sizeof(tmp_rel_path), file_name,
                            get_root_dir(), get_ensure_in_jail(),
                            FALSE))
    {
      log_error("file '%s' not under root directory '%s'\n", file_name,
                get_root_dir());
      err++;
      break;
    }
    else
    {
      file_name = tmp_rel_path;
    }

    if (get_verbosity())
    {
      printf("-- indexing '%s'\n", file_name);
    }

    /* do the main work in building the index */
    if (! build_unsorted_index(mode, type, file_name))
    {
      err++;
      break;
    }

    optind++;
  }

  if (err)
  {
    log_error("terminating due to errors");
    exit(1);
  }

  if (! flush_and_sort(in_jail))
  {
    log_error("terminating due to errors");
    exit(1);
  }
  
  if (get_verbosity())
  {
    printf("-- indexing sucessful!\n");
  }
}

static void
usage(prog_name)
  char  *prog_name;
{
  fprintf(stderr, "usage: %s [-c rwhois_conf_file] [-mi|-ma] [-tc|-ta] [-q] [-v] [-j] [-e] [files ...]\n", prog_name);
  fprintf(stderr, "       -c rwhois_conf_file: the path to the standard rwhois configuration\n");
  fprintf(stderr, "       -m<i|a>: mode -- <i>nitialize or <a>ppend\n");
  fprintf(stderr, "       -t<c|a>: type of data -- <c>ached or <a>uthoritative\n");
  fprintf(stderr, "       -q: quiet; surpress all diagnostic info\n");
  fprintf(stderr, "       -v: verbose; add additional diagnostic info\n");
  fprintf(stderr, "       -j: in jail; indexer is running in a chrooted env.\n");
  fprintf(stderr, "       -e: ensure in jail; rwhoisd will run in a chrooted env.\n");
  exit(64);
}



/* build_unsorted_index:  builds the index file - places in format
      offset:fileno:deleted:data
   and returns the number of records. */
int
build_unsorted_index(mode, type, data_file)
  int mode;
  int type;
  char *data_file;

{
  char  attribute_name[MAX_LINE];
  char  object_name[MAX_LINE];
  char  datum[MAX_LINE];
  char  line[MAX_LINE];
  long  offset  = 0;
  int   line_no = 0;
  int   file_no;
  FILE  *fp;
  FILE  *ifp;
  int   pass;
  int   do_increment_rec_count;

  
  if ((fp = fopen(data_file, "r")) == NULL) {
    log_error("build_unsorted_index: can not open data file '%s': %s",
              data_file, l_strerror(errno));
    return FALSE;
  }

  do_increment_rec_count = TRUE;
  *object_name           = '\0';

  while (fgets(line, MAX_LINE, fp) != NULL) {

    /* output a message every 1000 lines, if verbose */
    if (get_verbosity() && !(++line_no%1000)) {
      printf("** processed %d lines\n", line_no);
    }

    stripchar (line, '\n');

    if (record_separator(line)) {
      offset = ftell(fp);
      do_increment_rec_count = TRUE;
    }
    else if (get_tuple(object_name, attribute_name, datum, line) && 
             object_name[0] != '_') {
      if (! (ifp = get_object_db(object_name, data_file, mode, type, 
                                 &file_no)) ) {
        /* encountered an error trying to open the index file */
        return FALSE;
      }

      if (do_increment_rec_count) {
        increment_record_count(object_name, data_file);
        do_increment_rec_count = FALSE;
      }
      
      pass = 0;

      while (set_index_format(object_name, attribute_name, line, datum, 
                              pass++)) 
      {
        /* write the actual index entry */
        fprintf(ifp, "%ld:%d:%d:%s\n", offset, file_no, FALSE, line);
      }
    }
  }

  fclose (fp);

  return TRUE;
}

