/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "database.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "attributes.h"
#include "client_msgs.h"
#include "defines.h"
#include "llist.h"
#include "misc.h"
#include "objects.h"
#include "parse.h"
#include "read_config.h"
#include "validate.h"
#include "variables.h"

/* local macros */

#define DB_BASE_TEMPLATE        "%s.db"
#define INDEX_PARAMS            "-c %s -ma -ta -q"
#define INDEX_PROG              "rmkdbindex"
#define UPDATE_FILE_TEMPLATE    "%s/update.txt.XXXXXX"

/* local prototypes */

static int dump_record_list PROTO((char *filename, llist_struct *record_list));

static int append_mkdb_index PROTO((char *filename));

static int nuke_primary_key PROTO((llist_struct *record_list));

static int attach_index PROTO((char *attrib, char *value, char *buf));

static char *setup_index_program PROTO(());

/*************************************************************************
 updates the database
**************************************************************************/
int
update_database(action, record_list, filename)
  char          *action;
  llist_struct  *record_list;
  char          *filename;

{
  if (STR_EQ(action, "ADD"))
  {
    if (!dump_record_list(filename, record_list))
    {
      return (FALSE);
    }
    if (!append_mkdb_index(filename))
    {
      print_error(MISC_DIAG, "update_database: append to index failed\n");
      return (FALSE);
    }
  }
  else if (STR_EQ(action, "MOD"))
  {
    if (!dump_record_list(filename, record_list))
    {
      return (FALSE);
    }
    if (!nuke_primary_key(record_list))
    {
      unlink (filename);
      print_error (MISC_DIAG, "No primary key matches");
      return (FALSE);
    }
    if (!append_mkdb_index(filename) != 0)
    {
      print_error(MISC_DIAG, "update_database: append to index failed\n");
      return (FALSE);
    }
  }
  else if (STR_EQ(action, "DEL"))
  {
    if (!nuke_primary_key (record_list))
    {
      print_error(MISC_DIAG, "No primary key matches");
      return (FALSE);
    }
  }
  else
  {
    print_error (MISC_DIAG, "update_database: Unknown action |%s|", action);
    return (FALSE);
  }
  return (TRUE);
}

/*************************************************************************
 dumps the record list
  returns TRUE on sucess, FALSE on failure
**************************************************************************/
static int
dump_record_list(filename, record_list)
  char         *filename;
  llist_struct *record_list;

{
  FILE          *fp;
  llist_struct  *record;
  line_struct   *line_info;
  int           rec_cnt = 0;

  
  if ((fp = fopen(filename, "w")) == NULL)
  {
    print_error (MISC_DIAG, "dump_record_list: could not open |%s|", filename);
    return (FALSE);
  }

  while (record_list != NULL)
  {
    if (rec_cnt> 0)
    {
      fprintf (fp, "-----------------------------\n");
    }
    record = record_list->datum;
    while  (record != NULL)
    {
      line_info = record->datum;
      fprintf (fp, "%s\n", line_info->line);
      record = record->nxt;
    }
    rec_cnt++;
    record_list = record_list->nxt;
  }
  fclose (fp);
  return (TRUE);
}

/*************************************************************************
 appends a new index on the end of current user index
  returns TRUE on sucess, FALSE on failure
**************************************************************************/
static int
append_mkdb_index(filename)
  char *filename;
{
  extern int  in_jail;
  extern char *config_file;
  char        params[MAX_FILENAME*2];

  sprintf(params, INDEX_PARAMS, config_file);
  
  if (in_jail)
  {
    strcat(params, " -j");
  }

  strcat(params, " ");
  strcat(params,  filename);
  
  run_program(setup_index_program(), params);

  return (TRUE);
}

/*************************************************************************
  now for the tricky part - find the primary key in the index for
  each record, marks for delete all references to the offset and file no.
  returns TRUE on success, FALSE on failure
**************************************************************************/
static int
nuke_primary_key(record_list)
  llist_struct *record_list;
{
  char             object_name[MAX_LINE];
  char             attribute_name[MAX_LINE];
  char             datum[MAX_LINE];
  char             index_val[MAX_LINE];
  char             basefname[MAX_FILENAME];
  attribute_struct *attrib = NULL;
  object_struct    *object;
  llist_struct     *record;
  line_struct      *line_info;

  while (record_list != NULL)
  {
    record = record_list->datum;
    
    while  (record != NULL)
    {
      line_info = record->datum;
      /* don't have to do any checking here since done in validate_record */
      get_tuple(object_name, attribute_name, datum, line_info->line);
      attrib = link_object_to_attribute(object_name, attribute_name);
      if (attrib->primary_key)
      {
        break;
      }

      record = record->nxt;
    }

    if (record!= NULL)
    {
      attach_index(attrib->name, datum, index_val);
      object = get_object_by_object_name(object_name);
      sprintf(basefname, DB_BASE_TEMPLATE, object->local_db);
/*       if (!mkdb_delete_record(basefname, index_val)) */
/*       { */
/*         print_error (MISC_DIAG, "nuke_primary_key: could not find db\n"); */
/*         return (FALSE); */
/*       } */
    }
    else
    {
      print_error (MISC_DIAG, "nuke_primary_key: can not find primary key!");
      return (FALSE);
    }
    record_list = record_list->nxt;
  }
  return (TRUE);
}

/****************************************************************************
 attaches the unique index id's to look for
 returns FALSE if not found or no attrib/val combos
****************************************************************************/
static int
attach_index(attrib, value, buf)
  char *attrib;
  char *value;
  char *buf;
{
  int index_id;

  if ((index_id =  get_attribute_global_id_from_txt (attrib)) == -1)
  {
    return (FALSE);
  }

  sprintf (buf,"%d%s%s", index_id, INDEX_PRFX, value);

  return (TRUE);
}

static char *
setup_index_program()
{
  static char   prog[MAX_LINE];
  char          *bin_path = get_bin_path();
  
  if (*bin_path)
  {
    sprintf(prog, "%s/%s", bin_path, get_index_prog());
  }
  else
  {
    strcpy(prog, get_index_prog());
  }
  
  return(prog);
}

